;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "ansible" "0.4.3"
  "Ansible minor mode."
  '((s     "1.9.0")
    (f     "0.16.2")
    (emacs "25.1"))
  :url "https://gitlab.com/emacs-ansible/emacs-ansible"
  :commit "0d7bc93ad963677880d99c846a30ea6e6ed9eec5"
  :revdesc "0d7bc93ad963"
  :authors '((nil . "k1lowxb[at]gmail[dot]com")
             (nil . "k1low[at]101000lab[dot]org"))
  :maintainers '((nil . "k1lowxb[at]gmail[dot]com")
                 (nil . "k1low[at]101000lab[dot]org")))
